@@ -1,22 +1,21 @@
-use std::collections::HashMap;
-use std::env::args;
-use wmi::{COMLibrary, Variant, WMIConnection};
+use std::{collections::HashMap, env::args};
+use wmi::{Variant, WMIConnection, WMIResult};
 
-fn main() {
-    let wmi_con = WMIConnection::new(COMLibrary::new().unwrap().into()).unwrap();
+fn main() -> WMIResult<()> {
+    let wmi_con = WMIConnection::new()?;
     let args: Vec<String> = args().collect();
     let query = match args.get(1) {
         None => {
             println!("Expected an argument with a WMI query");
-            return;
+            return Ok(());
         }
         Some(query) => query,
     };
 
-    let results: Vec<HashMap<String, Variant>> = match wmi_con.raw_query(&query) {
+    let results: Vec<HashMap<String, Variant>> = match wmi_con.raw_query(query) {
         Err(e) => {
             println!("Couldn't run query {} because of {:?}", query, e);
-            return;
+            return Ok(());
         }
         Ok(results) => results,
     };
@@ -25,4 +24,6 @@ fn main() {
         println!("Result {}", i);
         println!("{:#?}", res);
     }
+
+    Ok(())
 }
