@@ -1,11 +1,11 @@
-use serde::de::{self, Deserialize, Deserializer, Visitor};
+use serde::de::{self, Deserialize, Deserializer, Visitor, value::Error};
 use serde::forward_to_deserialize_any;
 
 /// Return the fields of a struct.
 /// Taken directly from <https://github.com/serde-rs/serde/issues/1110>
 ///
-pub fn struct_name_and_fields<'de, T>(
-) -> Result<(&'static str, &'static [&'static str]), serde::de::value::Error>
+pub fn struct_name_and_fields<'de, T>()
+-> Result<(&'static str, Option<&'static [&'static str]>), Error>
 where
     T: Deserialize<'de>,
 {
@@ -15,7 +15,7 @@ where
     }
 
     impl<'de, 'a> Deserializer<'de> for StructNameAndFieldsDeserializer<'a> {
-        type Error = serde::de::value::Error;
+        type Error = Error;
 
         fn deserialize_any<V>(self, _visitor: V) -> Result<V::Value, Self::Error>
         where
@@ -26,12 +26,13 @@ where
 
         fn deserialize_newtype_struct<V>(
             self,
-            _name: &'static str,
+            name: &'static str,
             visitor: V,
         ) -> Result<V::Value, Self::Error>
         where
             V: Visitor<'de>,
         {
+            *self.name = Some(name);
             visitor.visit_newtype_struct(self)
         }
 
@@ -51,9 +52,20 @@ where
 
         forward_to_deserialize_any! {
             bool i8 i16 i32 i64 u8 u16 u32 u64 f32 f64 char str string bytes
-            byte_buf option unit unit_struct seq tuple
+            byte_buf option unit seq tuple
             tuple_struct map enum identifier ignored_any
         }
+
+        fn deserialize_unit_struct<V>(
+            self,
+            name: &'static str,
+            visitor: V,
+        ) -> Result<V::Value, Self::Error>
+        where
+            V: Visitor<'de>,
+        {
+            self.deserialize_struct(name, &[], visitor)
+        }
     }
 
     let mut name = None;
@@ -65,15 +77,17 @@ where
     });
 
     match name {
-        None =>  Err(de::Error::custom("Expected a named struct. \
-            Hint: You cannot use a HashMap<...> in this context because it requires the struct to have a name")),
+        None => Err(de::Error::custom(
+            "Expected a named struct. \
+            Hint: You cannot use a HashMap<...> in this context because it requires the struct to have a name",
+        )),
         Some(name) => {
             validate_identifier(name)?;
             for field in fields.into_iter().flatten() {
                 validate_identifier(field)?;
             }
 
-            Ok((name, fields.unwrap()))
+            Ok((name, fields))
         }
     }
 }
@@ -88,7 +102,7 @@ where
 /// > All following characters must be in set S2 where S2 = S1 union {U+0030...U+0039} \[This is alphabetic, underscore, plus Arabic numerals 0 through 9.\]<br>
 ///
 /// [DMTF-DSP0004]:     https://www.dmtf.org/sites/default/files/standards/documents/DSP0004V2.3_final.pdf
-fn validate_identifier<E: serde::de::Error>(s: &str) -> Result<&str, E> {
+fn validate_identifier<E: de::Error>(s: &str) -> Result<&str, E> {
     fn is_s1(ch: char) -> bool {
         match ch {
             '\u{005f}' => true,
@@ -122,6 +136,7 @@ fn validate_identifier<E: serde::de::Error>(s: &str) -> Result<&str, E> {
 }
 
 #[cfg(test)]
+#[allow(dead_code)]
 mod tests {
     use super::*;
     use crate::Variant;
@@ -132,16 +147,14 @@ mod tests {
     fn it_works() {
         #[derive(Deserialize, Debug)]
         struct Win32_OperatingSystem {
-            #[allow(dead_code)]
             Caption: String,
-            #[allow(dead_code)]
             Name: String,
         }
 
         let (name, fields) = struct_name_and_fields::<Win32_OperatingSystem>().unwrap();
 
         assert_eq!(name, "Win32_OperatingSystem");
-        assert_eq!(fields, ["Caption", "Name"]);
+        assert_eq!(fields.unwrap(), ["Caption", "Name"]);
     }
 
     #[test]
@@ -150,16 +163,34 @@ mod tests {
         #[serde(rename = "Win32_OperatingSystem")]
         #[serde(rename_all = "PascalCase")]
         struct Win32OperatingSystem {
-            #[allow(dead_code)]
             caption: String,
-            #[allow(dead_code)]
             name: String,
         }
 
         let (name, fields) = struct_name_and_fields::<Win32OperatingSystem>().unwrap();
 
         assert_eq!(name, "Win32_OperatingSystem");
-        assert_eq!(fields, ["Caption", "Name"]);
+        assert_eq!(fields.unwrap(), ["Caption", "Name"]);
+    }
+
+    #[test]
+    fn it_works_with_flatten() {
+        #[derive(Deserialize, Debug)]
+        struct Win32_OperatingSystem_inner {
+            Caption: String,
+            Name: String,
+
+            #[serde(flatten)]
+            extra: HashMap<String, Variant>,
+        }
+
+        #[derive(Deserialize, Debug)]
+        struct Win32_OperatingSystem(pub Win32_OperatingSystem_inner);
+
+        let (name, fields) = struct_name_and_fields::<Win32_OperatingSystem>().unwrap();
+
+        assert_eq!(name, "Win32_OperatingSystem");
+        assert_eq!(fields, None);
     }
 
     #[test]
@@ -171,7 +202,6 @@ mod tests {
         #[derive(Deserialize, Debug)]
         struct EvilFieldName {
             #[serde(rename = "Evil\"Field\"Name")]
-            #[allow(dead_code)]
             field: String,
         }
 
