@@ -1,11 +1,9 @@
-use std::collections::HashMap;
-
-use serde::{de, Serialize};
+use serde::{Serialize, de};
 use windows::core::BSTR;
 
 use crate::{
-    de::meta::struct_name_and_fields, result_enumerator::IWbemClassWrapper,
-    ser::variant_ser::VariantStructSerializer, Variant, WMIConnection, WMIError, WMIResult,
+    Variant, WMIConnection, WMIError, WMIResult, de::meta::struct_name_and_fields,
+    result_enumerator::IWbemClassWrapper, ser::variant_ser::VariantSerializer,
 };
 
 impl WMIConnection {
@@ -14,79 +12,41 @@ impl WMIConnection {
     /// This function is used internally by [`WMIConnection::exec_class_method`] and [`WMIConnection::exec_instance_method`],
     /// which are a higher-level abstraction, dealing with Rust data types instead of raw Variants, that should be preferred to use.
     ///
-    /// In the case of a class ("static") method, `object_path` should be the same as `method_class`.
+    /// In the case of a class ("static") method, `object_path` should be name of the class.
     ///
     /// Returns `None` if the method has no out parameters and a `void` return type, and an [`IWbemClassWrapper`] containing the output otherwise.
     /// A method with a return type other than `void` will always have a generic property named `ReturnValue` in the output class wrapper with the return value of the WMI method call.
     ///
     /// ```edition2021
-    /// # use wmi::{COMLibrary, Variant, WMIConnection, WMIResult};
+    /// # use std::collections::HashMap;
+    /// # use wmi::{Variant, WMIConnection, WMIResult};
     /// # fn main() -> WMIResult<()> {
-    /// # let wmi_con = WMIConnection::new(COMLibrary::new()?)?;
-    /// let in_params = [
-    ///     ("CommandLine".to_string(), Variant::from("explorer.exe".to_string()))
-    /// ].into_iter().collect();
+    /// # let wmi_con = WMIConnection::new()?;
+    /// let in_params = wmi_con
+    ///     .get_object("Win32_Process")?
+    ///     .get_method("Create")?
+    ///     .unwrap()
+    ///     .spawn_instance()?;
+    /// in_params.put_property("CommandLine", "explorer.exe".to_string())?;
     ///
     /// // Because Create has a return value and out parameters, the Option returned will never be None.
     /// // Note: The Create call can be unreliable, so consider using another means of starting processes.
-    /// let out = wmi_con.exec_method_native_wrapper("Win32_Process", "Win32_Process", "Create", in_params)?.unwrap();
+    /// let out = wmi_con.exec_method("Win32_Process", "Create", Some(&in_params))?.unwrap();
     /// println!("The return code of the Create call is {:?}", out.get_property("ReturnValue")?);
     /// # Ok(())
     /// # }
     /// ```
-    pub fn exec_method_native_wrapper(
+    pub fn exec_method(
         &self,
-        method_class: impl AsRef<str>,
         object_path: impl AsRef<str>,
         method: impl AsRef<str>,
-        in_params: HashMap<String, Variant>,
+        in_params: Option<&IWbemClassWrapper>,
     ) -> WMIResult<Option<IWbemClassWrapper>> {
-        let method_class = BSTR::from(method_class.as_ref());
         let object_path = BSTR::from(object_path.as_ref());
-        let method = BSTR::from(method.as_ref());
-
-        // See https://learn.microsoft.com/en-us/windows/win32/api/wbemcli/nf-wbemcli-iwbemclassobject-getmethod
-        // GetMethod can only be called on a class definition, so we retrieve that before retrieving a specific object
-        let mut class_definition = None;
-        unsafe {
-            self.svc.GetObject(
-                &method_class,
-                Default::default(),
-                &self.ctx.0,
-                Some(&mut class_definition),
-                None,
-            )?;
-        }
-        let class_definition = class_definition.ok_or(WMIError::ResultEmpty)?;
-        // Retrieve the input signature of the WMI method.
-        // The fields of the resulting IWbemClassObject will have the names and types of the WMI method's input parameters
-        let mut input_signature = None;
-        unsafe {
-            class_definition.GetMethod(
-                &method,
-                Default::default(),
-                &mut input_signature,
-                std::ptr::null_mut(),
-            )?;
-        }
-
-        // The method may have no input parameters, such as in this case: https://learn.microsoft.com/en-us/windows/win32/cimwin32prov/reboot-method-in-class-win32-operatingsystem
-        let in_params = match input_signature {
-            Some(input) => {
-                let inst = unsafe { input.SpawnInstance(Default::default())? };
-                let inst = IWbemClassWrapper::new(inst);
-
-                // Set every field of the input object to the corresponding input parameter passed to this function
-                for (name, value) in in_params {
-                    inst.put_property(&name, value)?;
-                }
-
-                Some(inst.inner)
-            }
-            None => None,
-        };
 
         // In the case of a method with no out parameters and a VOID return type, there will be no out-parameters object
+        let method = BSTR::from(method.as_ref());
+
         let mut output = None;
         unsafe {
             self.svc.ExecMethod(
@@ -94,7 +54,7 @@ impl WMIConnection {
                 &method,
                 Default::default(),
                 &self.ctx.0,
-                in_params.as_ref(),
+                in_params.as_ref().map(|param| &param.inner),
                 Some(&mut output),
                 None,
             )?;
@@ -107,19 +67,16 @@ impl WMIConnection {
     /// [Create](https://learn.microsoft.com/en-us/windows/win32/cimwin32prov/create-method-in-class-win32-process) of `Win32_Process`
     /// and [AddPrinterConnection](https://learn.microsoft.com/en-us/windows/win32/cimwin32prov/addprinterconnection-method-in-class-win32-printer) of `Win32_Printer`.
     ///
-    /// `MethodClass` should have the name of the class on which the method is being invoked.
-    /// `In` and `Out` can be `()` or any custom structs supporting (de)serialization containing the input and output parameters of the function.
-    ///
     /// A method with a return type other than `void` will always try to populate a generic property named `ReturnValue` in the output object with the return value of the WMI method call.
     /// If the method call has a `void` return type and no out parameters, the only acceptable type for `Out` is `()`.
     ///
     /// Arrays, Options, unknowns, and nested objects cannot be passed as input parameters due to limitations in how variants are constructed by `windows-rs`.
     ///
-    /// This function uses [`WMIConnection::exec_instance_method`] internally, with the name of the method class being the instance path, as is expected by WMI.
+    /// This function uses [`WMIConnection::exec_method`] internally, with the name of the method class being the instance path, as is expected by WMI.
     ///
     /// ```edition2021
     /// # use serde::{Deserialize, Serialize};
-    /// # use wmi::{COMLibrary, Variant, WMIConnection, WMIResult};
+    /// # use wmi::{Variant, WMIConnection, WMIResult};
     /// #[derive(Serialize)]
     /// # #[allow(non_snake_case)]
     /// struct CreateInput {
@@ -138,38 +95,36 @@ impl WMIConnection {
     /// struct Win32_Process;
     ///
     /// # fn main() -> WMIResult<()> {
-    /// # let wmi_con = WMIConnection::new(COMLibrary::new()?)?;
+    /// # let wmi_con = WMIConnection::new()?;
     /// // Note: The Create call can be unreliable, so consider using another means of starting processes.
     /// let input = CreateInput {
     ///     CommandLine: "explorer.exe".to_string()
     /// };
-    /// let output: CreateOutput = wmi_con.exec_class_method::<Win32_Process, _, _>("Create", input)?;
+    /// let output: CreateOutput = wmi_con
+    ///     .exec_class_method::<Win32_Process, _>("Create", input)?;
     ///
     /// println!("The return code of the Create call is {}", output.ReturnValue);
     /// println!("The ID of the created process is: {}", output.ProcessId);
     /// # Ok(())
     /// # }
     /// ```
-    pub fn exec_class_method<MethodClass, In, Out>(
+    pub fn exec_class_method<Class, Out>(
         &self,
         method: impl AsRef<str>,
-        in_params: In,
+        in_params: impl Serialize,
     ) -> WMIResult<Out>
     where
-        MethodClass: de::DeserializeOwned,
-        In: Serialize,
+        Class: de::DeserializeOwned,
         Out: de::DeserializeOwned,
     {
-        let (method_class, _) = struct_name_and_fields::<MethodClass>()?;
-        self.exec_instance_method::<MethodClass, In, Out>(method, method_class, in_params)
+        let (class, _) = struct_name_and_fields::<Class>()?;
+        self.exec_instance_method::<Class, _>(class, method, in_params)
     }
 
     /// Executes a WMI method on a specific instance of a class. Examples include
     /// [GetSupportedSize](https://learn.microsoft.com/en-us/windows-hardware/drivers/storage/msft-Volume-getsupportedsizes) of `MSFT_Volume`
     /// and [Pause](https://learn.microsoft.com/en-us/windows/win32/cimwin32prov/pause-method-in-class-win32-printer) of `Win32_Printer`.
     ///
-    /// `MethodClass` should have the name of the class on which the method is being invoked.
-    /// `In` and `Out` can be `()` or any custom structs supporting (de)serialization containing the input and output parameters of the function.
     /// `object_path` is the `__Path` variable of the class instance on which the method is being called, which can be obtained from a WMI query.
     ///
     /// A method with a return type other than `void` will always try to populate a generic property named `ReturnValue` in the output object with the return value of the WMI method call.
@@ -179,7 +134,7 @@ impl WMIConnection {
     ///
     /// ```edition2021
     /// # use serde::{Deserialize, Serialize};
-    /// # use wmi::{COMLibrary, FilterValue, Variant, WMIConnection, WMIResult};
+    /// # use wmi::{FilterValue, Variant, WMIConnection, WMIResult};
     /// #[derive(Deserialize)]
     /// # #[allow(non_snake_case)]
     /// struct PrinterOutput {
@@ -193,49 +148,69 @@ impl WMIConnection {
     /// }
     ///
     /// # fn main() -> WMIResult<()> {
-    /// # let wmi_con = WMIConnection::new(COMLibrary::new()?)?;
+    /// # let wmi_con = WMIConnection::new()?;
     /// let printers: Vec<Win32_Printer> = wmi_con.query()?;
     ///
     /// for printer in printers {
-    ///     let output: PrinterOutput = wmi_con.exec_instance_method::<Win32_Printer, _, _>("Pause", &printer.__Path, ())?;
+    ///     let output: PrinterOutput = wmi_con.exec_instance_method::<Win32_Printer, _>(&printer.__Path, "Pause", ())?;
     ///     println!("Pausing the printer returned {}", output.ReturnValue);
     ///
-    ///     let output: PrinterOutput = wmi_con.exec_instance_method::<Win32_Printer, _, _>("Resume", &printer.__Path, ())?;
+    ///     let output: PrinterOutput = wmi_con.exec_instance_method::<Win32_Printer, _>(&printer.__Path, "Resume", ())?;
     ///     println!("Resuming the printer returned {}", output.ReturnValue);
     /// }
     /// # Ok(())
     /// # }
     /// ```
-    pub fn exec_instance_method<MethodClass, In, Out>(
+    pub fn exec_instance_method<Class, Out>(
         &self,
-        method: impl AsRef<str>,
         object_path: impl AsRef<str>,
-        in_params: In,
+        method: impl AsRef<str>,
+        in_params: impl Serialize,
     ) -> WMIResult<Out>
     where
-        MethodClass: de::DeserializeOwned,
-        In: Serialize,
+        Class: de::DeserializeOwned,
         Out: de::DeserializeOwned,
     {
-        let (method_class, _) = struct_name_and_fields::<MethodClass>()?;
-        let serializer = VariantStructSerializer::new();
-        match in_params.serialize(serializer) {
-            Ok(field_map) => {
-                let output =
-                    self.exec_method_native_wrapper(method_class, object_path, method, field_map)?;
+        let (class, _) = struct_name_and_fields::<Class>()?;
+        let method = method.as_ref();
 
-                match output {
-                    Some(class_wrapper) => Ok(class_wrapper.into_desr()?),
-                    None => Out::deserialize(Variant::Empty),
+        // See https://learn.microsoft.com/en-us/windows/win32/api/wbemcli/nf-wbemcli-iwbemclassobject-getmethod
+        // GetMethod can only be called on a class definition, so we retrieve that before retrieving a specific object.
+        let instance = match self.get_object(class)?.get_method(method)? {
+            None => None,
+            Some(method_class) => {
+                let instance = method_class.spawn_instance()?;
+
+                let serializer = VariantSerializer {
+                    wmi: self,
+                    instance: Some(instance),
+                };
+
+                match in_params.serialize(serializer) {
+                    Ok(Variant::Object(instance)) => Some(instance),
+                    Ok(other) => {
+                        return Err(WMIError::ConvertVariantError(format!(
+                            "Unexpected serializer output: {:?}",
+                            other
+                        )));
+                    }
+                    Err(e) => return Err(WMIError::ConvertVariantError(e.to_string())),
                 }
             }
-            Err(e) => Err(WMIError::ConvertVariantError(e.to_string())),
+        };
+
+        let output = self.exec_method(object_path, method, instance.as_ref())?;
+
+        match output {
+            Some(class_wrapper) => Ok(class_wrapper.into_desr()?),
+            None => Out::deserialize(Variant::Empty),
         }
     }
 }
 
 #[cfg(test)]
 mod tests {
+    use crate::Variant;
     use crate::tests::fixtures::wmi_con;
     use serde::{Deserialize, Serialize};
     use std::thread::sleep;
@@ -244,28 +219,65 @@ mod tests {
     #[derive(Deserialize)]
     struct Win32_Process {
         __Path: String,
+        HandleCount: u32,
+    }
+
+    #[derive(Debug, Serialize, Default)]
+    pub struct Win32_ProcessStartup {
+        CreateFlags: u32,
     }
 
     #[derive(Serialize)]
-    struct CreateParams {
+    struct CreateInput {
         CommandLine: String,
+        ProcessStartupInformation: Win32_ProcessStartup,
     }
 
     #[derive(Deserialize)]
-    #[allow(non_snake_case)]
     struct CreateOutput {
         ReturnValue: u32,
         ProcessId: u32,
     }
 
+    #[test]
+    fn it_exec_methods_native() {
+        let wmi_con = wmi_con();
+
+        let in_params = wmi_con
+            .get_object("Win32_Process")
+            .unwrap()
+            .get_method("Create")
+            .unwrap()
+            .unwrap()
+            .spawn_instance()
+            .unwrap();
+
+        in_params
+            .put_property("CommandLine", "explorer.exe".to_string())
+            .unwrap();
+
+        let out = wmi_con
+            .exec_method("Win32_Process", "Create", Some(&in_params))
+            .unwrap();
+
+        let return_value = out.unwrap().get_property("ReturnValue").unwrap();
+
+        assert!(matches!(return_value, Variant::UI4(0)));
+    }
+
     #[test]
     fn it_exec_methods() {
         let wmi_con = wmi_con();
-        let in_params = CreateParams {
+        const CREATE_SUSPENDED: u32 = 4;
+
+        let in_params = CreateInput {
             CommandLine: "explorer.exe".to_string(),
+            ProcessStartupInformation: Win32_ProcessStartup {
+                CreateFlags: CREATE_SUSPENDED,
+            },
         };
-        let out = wmi_con
-            .exec_class_method::<Win32_Process, CreateParams, CreateOutput>("Create", in_params)
+        let out: CreateOutput = wmi_con
+            .exec_class_method::<Win32_Process, _>("Create", &in_params)
             .unwrap();
 
         assert_eq!(out.ReturnValue, 0);
@@ -277,8 +289,11 @@ mod tests {
 
         let process = &wmi_con.raw_query::<Win32_Process>(&query).unwrap()[0];
 
-        wmi_con
-            .exec_instance_method::<Win32_Process, (), ()>("Terminate", &process.__Path, ())
+        // Since we started the process as suspended, it will not have any open handles.
+        assert_eq!(process.HandleCount, 0);
+
+        let _: () = wmi_con
+            .exec_instance_method::<Win32_Process, _>(&process.__Path, "Terminate", ())
             .unwrap();
 
         // It can take a moment for the process to terminate, so we retry the query a few times.
@@ -299,8 +314,8 @@ mod tests {
         #[derive(Deserialize)]
         struct StdRegProv;
 
-        #[derive(Serialize)]
-        struct GetBinaryValueParams {
+        #[derive(Deserialize, Serialize)]
+        struct GetBinaryValue {
             sSubKeyName: String,
             sValueName: String,
         }
@@ -310,15 +325,103 @@ mod tests {
             uValue: Vec<u8>,
         }
 
-        let in_params = GetBinaryValueParams {
+        let get_binary_value_params = GetBinaryValue {
             sSubKeyName: r#"SYSTEM\CurrentControlSet\Control\Windows"#.to_string(),
             sValueName: "FullProcessInformationSID".to_string(),
         };
 
         let value: GetBinaryValueOut = wmi_con
-            .exec_class_method::<StdRegProv, _, _>("GetBinaryValue", in_params)
+            .exec_class_method::<StdRegProv, _>("GetBinaryValue", &get_binary_value_params)
             .unwrap();
 
         assert!(value.uValue.len() > 0, "Expected to get a non-empty value");
+
+        #[derive(Deserialize, Serialize)]
+        struct SetBinaryValue {
+            sSubKeyName: String,
+            sValueName: String,
+            uValue: Vec<u8>,
+        }
+
+        #[derive(Deserialize)]
+        struct SetBinaryValueOut {
+            ReturnValue: u32,
+        }
+
+        let test_value_name = format!("{}.test", get_binary_value_params.sValueName);
+        let test_value = vec![0, 1, 2, 3];
+
+        let set_binary_value_params = SetBinaryValue {
+            sSubKeyName: get_binary_value_params.sSubKeyName,
+            sValueName: test_value_name,
+            uValue: test_value,
+        };
+
+        let value: SetBinaryValueOut = wmi_con
+            .exec_class_method::<StdRegProv, _>("SetBinaryValue", &set_binary_value_params)
+            .unwrap();
+
+        assert_eq!(value.ReturnValue, 0);
+
+        let get_test_binary_value_params = GetBinaryValue {
+            sSubKeyName: set_binary_value_params.sSubKeyName,
+            sValueName: set_binary_value_params.sValueName,
+        };
+
+        let value: GetBinaryValueOut = wmi_con
+            .exec_class_method::<StdRegProv, _>("GetBinaryValue", &get_test_binary_value_params)
+            .unwrap();
+
+        assert_eq!(value.uValue, set_binary_value_params.uValue);
+
+        wmi_con
+            .exec_class_method::<StdRegProv, ()>("DeleteValue", &get_test_binary_value_params)
+            .unwrap();
+    }
+
+    #[test]
+    fn it_exec_with_object_arrays() {
+        let wmi_con = wmi_con();
+
+        #[derive(Deserialize)]
+        struct StdRegProv;
+
+        #[derive(Serialize)]
+        struct GetSecurityDescriptor {
+            sSubKeyName: String,
+        }
+
+        #[derive(Deserialize, Debug)]
+        struct __Trustee {
+            Name: String,
+        }
+
+        #[derive(Deserialize, Debug)]
+        struct __ACE {
+            Trustee: __Trustee,
+        }
+
+        #[derive(Deserialize, Debug)]
+        struct __SecurityDescriptor {
+            DACL: Vec<__ACE>,
+        }
+
+        #[derive(Deserialize, Debug)]
+        struct GetSecurityDescriptorOut {
+            Descriptor: Option<__SecurityDescriptor>,
+        }
+
+        let params = GetSecurityDescriptor {
+            sSubKeyName: r#"SECURITY"#.to_string(),
+        };
+
+        let value: GetSecurityDescriptorOut = wmi_con
+            .exec_class_method::<StdRegProv, _>("GetSecurityDescriptor", &params)
+            .unwrap();
+
+        let dacl = value.Descriptor.unwrap().DACL;
+
+        assert!(dacl.len() > 0, "Expected to get a non-empty value");
+        assert!(dacl.iter().find(|ac| ac.Trustee.Name == "SYSTEM").is_some());
     }
 }
