@@ -1,15 +1,14 @@
 use crate::{
-    build_notification_query,
+    FilterValue, WMIConnection, WMIResult, build_notification_query,
     query_sink::{AsyncQueryResultStream, AsyncQueryResultStreamInner, QuerySink},
     result_enumerator::{IWbemClassWrapper, QueryResultEnumerator},
-    FilterValue, WMIConnection, WMIResult,
 };
 use futures::{Stream, StreamExt};
 use std::{collections::HashMap, time::Duration};
-use windows::core::BSTR;
 use windows::Win32::System::Wmi::{
     IWbemObjectSink, WBEM_FLAG_FORWARD_ONLY, WBEM_FLAG_RETURN_IMMEDIATELY,
 };
+use windows::core::BSTR;
 
 ///
 /// ### Additional notification query methods
@@ -18,10 +17,10 @@ impl WMIConnection {
     /// Execute the given query to receive events and return an iterator of WMI pointers.
     /// It's better to use the other query methods, since this is relatively low level.
     ///
-    pub fn notification_native_wrapper(
+    pub fn exec_notification_query(
         &self,
         query: impl AsRef<str>,
-    ) -> WMIResult<QueryResultEnumerator> {
+    ) -> WMIResult<impl Iterator<Item = WMIResult<IWbemClassWrapper>> + 'static> {
         let query_language = BSTR::from("WQL");
         let query = BSTR::from(query.as_ref());
 
@@ -33,9 +32,8 @@ impl WMIConnection {
                 None,
             )?
         };
-        log::trace!("Got enumerator {:?}", enumerator);
 
-        Ok(QueryResultEnumerator::new(self, enumerator))
+        Ok(QueryResultEnumerator::new(enumerator))
     }
 
     /// Execute a free-text query and deserialize the incoming events.
@@ -53,19 +51,19 @@ impl WMIConnection {
     /// # }
     /// # fn run() -> wmi::WMIResult<()> {
     /// # use std::collections::HashMap;
-    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
+    /// # let con = WMIConnection::new()?;
     /// let iterator = con.raw_notification::<HashMap<String, Variant>>("SELECT ProcessID, ProcessName FROM Win32_ProcessStartTrace")?;
     /// #   Ok(()) // This query will fail when not run as admin
     /// # }
     /// ```
-    pub fn raw_notification<'a, T>(
-        &'a self,
+    pub fn raw_notification<T>(
+        &self,
         query: impl AsRef<str>,
-    ) -> WMIResult<impl Iterator<Item = WMIResult<T>> + 'a>
+    ) -> WMIResult<impl Iterator<Item = WMIResult<T>> + 'static>
     where
-        T: serde::de::DeserializeOwned + 'a,
+        T: serde::de::DeserializeOwned,
     {
-        let enumerator = self.notification_native_wrapper(query)?;
+        let enumerator = self.exec_notification_query(query)?;
         let iter = enumerator.map(|item| match item {
             Ok(wbem_class_obj) => wbem_class_obj.into_desr(),
             Err(e) => Err(e),
@@ -86,7 +84,7 @@ impl WMIConnection {
     /// # fn run() -> wmi::WMIResult<()> {
     /// use serde::Deserialize;
     ///
-    /// let con = WMIConnection::new(COMLibrary::new()?)?;
+    /// let con = WMIConnection::new()?;
     ///
     /// #[derive(Deserialize, Debug)]
     /// struct Win32_ProcessStartTrace {
@@ -98,9 +96,9 @@ impl WMIConnection {
     /// #   Ok(()) // This query will fail when not run as admin
     /// # }
     /// ```
-    pub fn notification<'a, T>(&'a self) -> WMIResult<impl Iterator<Item = WMIResult<T>> + 'a>
+    pub fn notification<T>(&self) -> WMIResult<impl Iterator<Item = WMIResult<T>> + 'static>
     where
-        T: serde::de::DeserializeOwned + 'a,
+        T: serde::de::DeserializeOwned,
     {
         let query_text = build_notification_query::<T>(None, None)?;
         self.raw_notification(query_text)
@@ -113,7 +111,7 @@ impl WMIConnection {
     /// # fn main() -> wmi::WMIResult<()> {
     /// # use std::{collections::HashMap, time::Duration};
     /// # use wmi::*;
-    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
+    /// # let con = WMIConnection::new()?;
     /// use serde::Deserialize;
     /// #[derive(Deserialize, Debug)]
     /// struct __InstanceCreationEvent {
@@ -133,13 +131,13 @@ impl WMIConnection {
     /// #   Ok(())
     /// # }
     /// ```
-    pub fn filtered_notification<'a, T>(
-        &'a self,
+    pub fn filtered_notification<T>(
+        &self,
         filters: &HashMap<String, FilterValue>,
         within: Option<Duration>,
-    ) -> WMIResult<impl Iterator<Item = WMIResult<T>> + 'a>
+    ) -> WMIResult<impl Iterator<Item = WMIResult<T>> + 'static>
     where
-        T: serde::de::DeserializeOwned + 'a,
+        T: serde::de::DeserializeOwned,
     {
         let query_text = build_notification_query::<T>(Some(filters), within)?;
         self.raw_notification(query_text)
@@ -149,7 +147,7 @@ impl WMIConnection {
     /// method. Provides safety checks, and returns results
     /// as a stream instead of the original Sink.
     ///
-    pub fn async_notification_native_wrapper(
+    pub fn exec_notification_query_async(
         &self,
         query: impl AsRef<str>,
     ) -> WMIResult<impl Stream<Item = WMIResult<IWbemClassWrapper>>> {
@@ -200,7 +198,7 @@ impl WMIConnection {
     /// # }
     /// #
     /// # async fn exec_async_query() -> WMIResult<()> {
-    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
+    /// # let con = WMIConnection::new()?;
     /// let mut stream = con.async_raw_notification::<HashMap<String, Variant>>("SELECT ProcessID, ProcessName FROM Win32_ProcessStartTrace")?;
     /// # let event = stream.next().await.unwrap()?;
     /// #   Ok(()) // This query will fail when not run as admin
@@ -214,7 +212,7 @@ impl WMIConnection {
         T: serde::de::DeserializeOwned,
     {
         let stream = self
-            .async_notification_native_wrapper(query)?
+            .exec_notification_query_async(query)?
             .map(|item| match item {
                 Ok(wbem_class_obj) => wbem_class_obj.into_desr(),
                 Err(e) => Err(e),
@@ -236,7 +234,7 @@ impl WMIConnection {
     /// # }
     /// #
     /// # async fn exec_async_query() -> WMIResult<()> {
-    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
+    /// # let con = WMIConnection::new()?;
     /// use futures::StreamExt;
     /// use serde::Deserialize;
     ///
@@ -277,7 +275,7 @@ impl WMIConnection {
     /// #
     /// # async fn exec_async_query() -> WMIResult<()> {
     /// # use std::{collections::HashMap, time::Duration};
-    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
+    /// # let con = WMIConnection::new()?;
     /// use futures::StreamExt;
     /// use serde::Deserialize;
     /// #[derive(Deserialize, Debug)]
@@ -315,7 +313,7 @@ impl WMIConnection {
 
 #[cfg(test)]
 mod tests {
-    use crate::{tests::fixtures::*, FilterValue, WMIError};
+    use crate::{FilterValue, WMIError, tests::fixtures::*};
     use futures::StreamExt;
     use serde::Deserialize;
     use std::{collections::HashMap, time::Duration};
@@ -354,7 +352,7 @@ mod tests {
     fn it_works() {
         let wmi_con = wmi_con();
 
-        let mut enumerator = wmi_con.notification_native_wrapper(TEST_QUERY).unwrap();
+        let mut enumerator = wmi_con.exec_notification_query(TEST_QUERY).unwrap();
 
         let res = enumerator.next().unwrap();
         let w = res.unwrap();
@@ -371,7 +369,7 @@ mod tests {
     fn it_fails_gracefully() {
         let wmi_con = wmi_con();
 
-        let mut enumerator = wmi_con.notification_native_wrapper("SELECT NoSuchField FROM __InstanceModificationEvent WHERE TargetInstance ISA 'Win32_LocalTime'").unwrap();
+        let mut enumerator = wmi_con.exec_notification_query("SELECT NoSuchField FROM __InstanceModificationEvent WHERE TargetInstance ISA 'Win32_LocalTime'").unwrap();
 
         let res = enumerator.next().unwrap();
         assert!(res.is_ok());
@@ -384,7 +382,7 @@ mod tests {
     fn it_fails_gracefully_with_invalid_sql() {
         let wmi_con = wmi_con();
 
-        let result = wmi_con.notification_native_wrapper("42");
+        let result = wmi_con.exec_notification_query("42");
 
         match result {
             Ok(_) => assert!(false),
@@ -476,7 +474,7 @@ mod tests {
         let wmi_con = wmi_con();
 
         let result = wmi_con
-            .async_notification_native_wrapper(TEST_QUERY)
+            .exec_notification_query_async(TEST_QUERY)
             .unwrap()
             .next()
             .await
@@ -490,7 +488,7 @@ mod tests {
         let wmi_con = wmi_con();
 
         let result = wmi_con
-            .async_notification_native_wrapper(TEST_QUERY)
+            .exec_notification_query_async(TEST_QUERY)
             .unwrap()
             .next()
             .await
@@ -503,7 +501,7 @@ mod tests {
     async fn async_it_handles_invalid_query() {
         let wmi_con = wmi_con();
 
-        let result = wmi_con.async_notification_native_wrapper("Invalid Query");
+        let result = wmi_con.exec_notification_query_async("Invalid Query");
 
         assert!(result.is_err());
         if let WMIError::HResultError { hres } = result.err().unwrap() {
