@@ -1,12 +1,43 @@
+use crate::safearray::SafeArrayAccessor;
 use crate::{
-    result_enumerator::IWbemClassWrapper, safearray::safe_array_to_vec, WMIError, WMIResult,
+    WMIError, WMIResult, result_enumerator::IWbemClassWrapper, safearray::safe_array_to_vec,
 };
 use serde::Serialize;
 use std::convert::TryFrom;
-use windows::core::{IUnknown, Interface};
+use std::ptr::NonNull;
 use windows::Win32::Foundation::{VARIANT_FALSE, VARIANT_TRUE};
+use windows::Win32::System::Ole::SafeArrayCreateVector;
 use windows::Win32::System::Variant::*;
-use windows::Win32::System::Wmi::{self, IWbemClassObject, CIMTYPE_ENUMERATION};
+use windows::Win32::System::Variant::{VARIANT, VT_NULL};
+use windows::Win32::System::Wmi::{self, CIMTYPE_ENUMERATION, IWbemClassObject};
+use windows::core::{BOOL, IUnknown, Interface, PCWSTR};
+
+fn variant_from_string_array(array: &[String]) -> WMIResult<VARIANT> {
+    // Convert the strings to null terminated vectors of `u16`s.
+    let v: Vec<Vec<u16>> = array
+        .iter()
+        .map(|s| s.encode_utf16().chain([0]).collect())
+        .collect();
+
+    // Safety: each pointer points to a null terminated slice of `u16`s.
+    let v: Vec<_> = v
+        .iter()
+        .map(|b| unsafe { PCWSTR::from_raw(b.as_ptr()) })
+        .collect();
+
+    // The new variant will allocate new `BSTR`s for the array,
+    // so it is safe to free `v` after this call.
+    let variant = unsafe { InitVariantFromStringArray(&v) }?;
+
+    Ok(variant)
+}
+
+fn set_variant_type(variant: &mut VARIANT, new_type: VARENUM) {
+    // Safety: it's always valid to access the `vt` field.
+    unsafe {
+        (&mut variant.Anonymous.Anonymous).vt = new_type;
+    }
+}
 
 #[derive(Debug, PartialEq, Serialize, Clone)]
 #[serde(untagged)]
@@ -84,15 +115,13 @@ impl Variant {
         // 1. A simple type like `VT_BSTR` .
         // 2. An array of certain type like `VT_ARRAY | VT_BSTR`.
         if variant_type & VT_ARRAY == VT_ARRAY {
-            let array = unsafe {
-                vt.Anonymous.Anonymous.Anonymous.parray
-                    as *const windows::Win32::System::Com::SAFEARRAY
-            };
+            let array = NonNull::new(unsafe { vt.Anonymous.Anonymous.Anonymous.parray })
+                .ok_or(WMIError::NullPointerResult)?;
 
             let item_type = variant_type & VT_TYPEMASK;
 
             return Ok(Variant::Array(unsafe {
-                safe_array_to_vec(&*array, item_type)?
+                safe_array_to_vec(array, item_type)?
             }));
         }
 
@@ -273,15 +302,23 @@ impl Variant {
 
 impl TryFrom<Variant> for VARIANT {
     type Error = WMIError;
+
     fn try_from(value: Variant) -> WMIResult<VARIANT> {
+        // Some rules are special cased by https://learn.microsoft.com/en-us/windows/win32/wmisdk/numbers.
+        // For int64, we also must use decimal and not hexadecimal:
+        // https://learn.microsoft.com/en-us/windows/win32/api/wbemcli/nf-wbemcli-iwbemclassobject-put#examples.
         match value {
             Variant::Empty => Ok(VARIANT::default()),
 
             Variant::String(string) => Ok(VARIANT::from(string.as_str())),
-            Variant::I1(int8) => Ok(VARIANT::from(int8)),
+
+            // sint8 uses VT_I2.
+            Variant::I1(int8) => Ok(VARIANT::from(int8 as i16)),
             Variant::I2(int16) => Ok(VARIANT::from(int16)),
             Variant::I4(int32) => Ok(VARIANT::from(int32)),
-            Variant::I8(int64) => Ok(VARIANT::from(int64)),
+
+            // Signed 64-bit integer in string form
+            Variant::I8(int64) => Ok(VARIANT::from(int64.to_string().as_str())),
 
             Variant::R4(float32) => Ok(VARIANT::from(float32)),
             Variant::R8(float64) => Ok(VARIANT::from(float64)),
@@ -289,23 +326,141 @@ impl TryFrom<Variant> for VARIANT {
             Variant::Bool(b) => Ok(VARIANT::from(b)),
 
             Variant::UI1(uint8) => Ok(VARIANT::from(uint8)),
-            Variant::UI2(uint16) => Ok(VARIANT::from(uint16)),
-            Variant::UI4(uint32) => Ok(VARIANT::from(uint32)),
-            Variant::UI8(uint64) => Ok(VARIANT::from(uint64)),
 
-            // windows-rs' VARIANT does not support creating these types of VARIANT at present
-            Variant::Null => Err(WMIError::ConvertVariantError(
-                "Cannot convert Variant::Null to a Windows VARIANT".to_string(),
-            )),
-            Variant::Array(_) => Err(WMIError::ConvertVariantError(
-                "Cannot convert Variant::Array to a Windows VARIANT".to_string(),
-            )),
-            Variant::Unknown(_) => Err(WMIError::ConvertVariantError(
-                "Cannot convert Variant::Unknown to a Windows VARIANT".to_string(),
-            )),
-            Variant::Object(_) => Err(WMIError::ConvertVariantError(
-                "Cannot convert Variant::Object to a Windows VARIANT".to_string(),
-            )),
+            // uint16 uses VT_I4.
+            Variant::UI2(uint16) => Ok(VARIANT::from(uint16 as i32)),
+            // uint32 uses VT_I4.
+            Variant::UI4(uint32) => Ok(VARIANT::from(uint32 as i32)),
+
+            // Signed 64-bit integer in string form.
+            Variant::UI8(uint64) => Ok(VARIANT::from(uint64.to_string().as_str())),
+
+            Variant::Object(instance) => Ok(VARIANT::from(IUnknown::from(instance.inner))),
+            Variant::Unknown(unknown) => Ok(VARIANT::from(unknown.inner)),
+
+            Variant::Null => {
+                let mut variant = VARIANT::default();
+                set_variant_type(&mut variant, VT_NULL);
+                Ok(variant)
+            }
+            Variant::Array(array) => {
+                // Variant arrays can only contain a single type, and we only support types that have utility functions in the `windows` crate.
+                match array.first() {
+                    // The "Empty" (default) variant is not a valid array.
+                    None => Ok(Variant::Null.try_into()?),
+                    Some(Variant::UI1(_)) => {
+                        let v: Vec<u8> = Variant::Array(array).try_into()?;
+
+                        //  "Creates a VT_ARRAY | VT_UI1 variant".
+                        let variant =
+                            unsafe { InitVariantFromBuffer(v.as_ptr() as _, v.len() as _) }?;
+                        Ok(variant)
+                    }
+                    Some(Variant::UI2(_)) => {
+                        let v: Vec<u16> = Variant::Array(array).try_into()?;
+
+                        // uint16 uses VT_I4.
+                        let v: Vec<i32> = v.into_iter().map(i32::from).collect();
+
+                        let variant = unsafe { InitVariantFromInt32Array(&v) }?;
+                        Ok(variant)
+                    }
+                    Some(Variant::UI4(_)) => {
+                        let v: Vec<u32> = Variant::Array(array).try_into()?;
+
+                        // uint32 uses VT_I4.
+                        let v: Vec<i32> = v.into_iter().map(|i| i as _).collect();
+
+                        let variant = unsafe { InitVariantFromInt32Array(&v) }?;
+                        Ok(variant)
+                    }
+                    Some(Variant::UI8(_)) => {
+                        let v: Vec<u64> = Variant::Array(array).try_into()?;
+
+                        // Unsigned 64-bit integer in string form.
+                        let v: Vec<String> = v.into_iter().map(|i| i.to_string()).collect();
+
+                        Ok(variant_from_string_array(&v)?)
+                    }
+                    Some(Variant::I1(_)) => {
+                        let v: Vec<i8> = Variant::Array(array).try_into()?;
+
+                        // sint8 uses VT_I2.
+                        let v: Vec<i16> = v.into_iter().map(i16::from).collect();
+
+                        let variant = unsafe { InitVariantFromInt16Array(&v) }?;
+                        Ok(variant)
+                    }
+                    Some(Variant::I2(_)) => {
+                        let v: Vec<i16> = Variant::Array(array).try_into()?;
+
+                        let variant = unsafe { InitVariantFromInt16Array(&v) }?;
+                        Ok(variant)
+                    }
+                    Some(Variant::I4(_)) => {
+                        let v: Vec<i32> = Variant::Array(array).try_into()?;
+
+                        let variant = unsafe { InitVariantFromInt32Array(&v) }?;
+                        Ok(variant)
+                    }
+                    Some(Variant::I8(_)) => {
+                        let v: Vec<i64> = Variant::Array(array).try_into()?;
+
+                        // Signed 64-bit integer in string form.
+                        let v: Vec<String> = v.into_iter().map(|i| i.to_string()).collect();
+
+                        Ok(variant_from_string_array(&v)?)
+                    }
+                    Some(Variant::R4(_)) => {
+                        let v: Vec<f32> = Variant::Array(array).try_into()?;
+
+                        let safe_arr =
+                            NonNull::new(unsafe { SafeArrayCreateVector(VT_R4, 0, v.len() as _) })
+                                .ok_or(WMIError::NullPointerResult)?;
+
+                        let mut accessor = unsafe { SafeArrayAccessor::new(safe_arr) }?;
+
+                        for (src, dst) in v.into_iter().zip(accessor.iter_mut()) {
+                            *dst = src;
+                        }
+
+                        drop(accessor);
+
+                        let mut variant = VARIANT::default();
+                        set_variant_type(&mut variant, VT_ARRAY | VT_R4);
+
+                        //  According to https://learn.microsoft.com/en-us/windows/win32/api/oleauto/nf-oleauto-variantclear:
+                        // "If the vt field has the VT_ARRAY bit set, the array is freed."
+                        // Therefore, we must not destroy the array ourselves, as the ownership is transferred to the variant.
+                        unsafe {
+                            (&mut variant.Anonymous.Anonymous).Anonymous.parray = safe_arr.as_ptr();
+                        }
+
+                        Ok(variant)
+                    }
+                    Some(Variant::R8(_)) => {
+                        let v: Vec<f64> = Variant::Array(array).try_into()?;
+
+                        let variant = unsafe { InitVariantFromDoubleArray(&v) }?;
+                        Ok(variant)
+                    }
+                    Some(Variant::Bool(_)) => {
+                        let v: Vec<bool> = Variant::Array(array).try_into()?;
+                        let v: Vec<_> = v.into_iter().map(BOOL::from).collect();
+
+                        let variant = unsafe { InitVariantFromBooleanArray(&v) }?;
+                        Ok(variant)
+                    }
+                    Some(Variant::String(_)) => {
+                        let v: Vec<String> = Variant::Array(array).try_into()?;
+
+                        Ok(variant_from_string_array(&v)?)
+                    }
+                    other => Err(WMIError::ConvertVariantError(format!(
+                        "Cannot convert {other:?} to a Windows VARIANT"
+                    ))),
+                }
+            }
         }
     }
 }
@@ -340,11 +495,53 @@ macro_rules! impl_wrap_type {
     };
 }
 
+macro_rules! impl_try_vec_from_variant {
+    ($target_type:ty, $variant_type:ident) => {
+        impl TryFrom<Variant> for Vec<$target_type> {
+            type Error = WMIError;
+
+            fn try_from(value: Variant) -> Result<Vec<$target_type>, Self::Error> {
+                let array = match value {
+                    Variant::Array(array) => array,
+                    _ => {
+                        return Err(WMIError::ConvertVariantError(format!(
+                            "Cannot convert a non Variant::Array {:?} to Vec",
+                            value
+                        )));
+                    }
+                };
+
+                let mut output_vec = Vec::with_capacity(array.len());
+
+                for item in array {
+                    let item = item.try_into()?;
+                    output_vec.push(item);
+                }
+
+                Ok(output_vec)
+            }
+        }
+    };
+}
+
+/// Infallible conversion from a Rust type into a Variant wrapper for that type
+macro_rules! impl_wrap_vec_type {
+    ($target_type:ty, $variant_type:ident) => {
+        impl From<Vec<$target_type>> for Variant {
+            fn from(value: Vec<$target_type>) -> Self {
+                Variant::Array(value.into_iter().map(Variant::$variant_type).collect())
+            }
+        }
+    };
+}
+
 /// Add conversions from a Rust type to its Variant form and vice versa
 macro_rules! bidirectional_variant_convert {
     ($target_type:ty, $variant_type:ident) => {
         impl_try_from_variant!($target_type, $variant_type);
+        impl_try_vec_from_variant!($target_type, $variant_type);
         impl_wrap_type!($target_type, $variant_type);
+        impl_wrap_vec_type!($target_type, $variant_type);
     };
 }
 
@@ -360,6 +557,7 @@ bidirectional_variant_convert!(u64, UI8);
 bidirectional_variant_convert!(f32, R4);
 bidirectional_variant_convert!(f64, R8);
 bidirectional_variant_convert!(bool, Bool);
+bidirectional_variant_convert!(IWbemClassWrapper, Object);
 
 impl From<()> for Variant {
     fn from(_value: ()) -> Self {
@@ -389,7 +587,7 @@ impl TryFrom<Variant> for () {
 }
 
 /// A wrapper around the [`IUnknown`] interface. \
-/// Used to retrive [`IWbemClassObject`][winapi::um::Wmi::IWbemClassObject]
+/// Used to retrieve [`IWbemClassObject`][winapi::um::Wmi::IWbemClassObject]
 ///
 #[repr(transparent)]
 #[derive(Debug, PartialEq, Eq, Clone)]
@@ -398,7 +596,7 @@ pub struct IUnknownWrapper {
 }
 
 impl IUnknownWrapper {
-    /// Wrapps around a non-null pointer to IUnknown
+    /// Wraps a non-null pointer to IUnknown
     ///
     pub fn new(ptr: IUnknown) -> Self {
         IUnknownWrapper { inner: ptr }
@@ -424,6 +622,8 @@ impl Serialize for IUnknownWrapper {
 
 #[cfg(test)]
 mod tests {
+    use windows::Win32::System::Wmi::{CIM_SINT8, CIM_SINT64, CIM_UINT16, CIM_UINT32, CIM_UINT64};
+
     use super::*;
 
     #[test]
@@ -665,4 +865,139 @@ mod tests {
         let variant = Variant::from(num);
         assert_eq!(Variant::from_variant(&ms_variant).unwrap(), variant);
     }
+
+    #[test]
+    fn it_convert_array_to_vec() {
+        let v: Vec<u8> = Variant::Array(vec![Variant::UI1(1), Variant::UI1(2)])
+            .try_into()
+            .unwrap();
+
+        assert_eq!(v, vec![1, 2]);
+
+        let _v: Vec<u16> = Variant::Array(vec![Variant::UI2(1)]).try_into().unwrap();
+        let _v: Vec<u32> = Variant::Array(vec![Variant::UI4(1)]).try_into().unwrap();
+        let _v: Vec<u64> = Variant::Array(vec![Variant::UI8(1)]).try_into().unwrap();
+
+        let _v: Vec<i8> = Variant::Array(vec![Variant::I1(1)]).try_into().unwrap();
+        let _v: Vec<i16> = Variant::Array(vec![Variant::I2(1)]).try_into().unwrap();
+        let _v: Vec<i32> = Variant::Array(vec![Variant::I4(1)]).try_into().unwrap();
+        let _v: Vec<i64> = Variant::Array(vec![Variant::I8(1)]).try_into().unwrap();
+
+        let _v: Vec<f32> = Variant::Array(vec![Variant::R4(1.)]).try_into().unwrap();
+        let _v: Vec<f64> = Variant::Array(vec![Variant::R8(1.)]).try_into().unwrap();
+
+        let _v: Vec<String> = Variant::Array(vec![Variant::String("s".to_string())])
+            .try_into()
+            .unwrap();
+        let _v: Vec<bool> = Variant::Array(vec![Variant::Bool(true)])
+            .try_into()
+            .unwrap();
+    }
+
+    #[test]
+    fn it_convert_array_to_ms_variant() {
+        let variant = Variant::Array(vec![Variant::UI1(1), Variant::UI1(2)]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant).unwrap();
+
+        assert_eq!(variant, converted_back_variant);
+
+        let variant = Variant::Array(vec![Variant::UI2(1), Variant::UI2(2)]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant)
+            .unwrap()
+            .convert_into_cim_type(CIM_UINT16)
+            .unwrap();
+
+        assert_eq!(variant, converted_back_variant);
+
+        let variant = Variant::Array(vec![Variant::UI4(1), Variant::UI4(2)]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant)
+            .unwrap()
+            .convert_into_cim_type(CIM_UINT32)
+            .unwrap();
+
+        assert_eq!(variant, converted_back_variant);
+
+        let variant = Variant::Array(vec![Variant::UI8(1), Variant::UI8(2)]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant)
+            .unwrap()
+            .convert_into_cim_type(CIM_UINT64)
+            .unwrap();
+
+        assert_eq!(variant, converted_back_variant);
+
+        let variant = Variant::Array(vec![Variant::I2(1), Variant::I2(2)]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant).unwrap();
+
+        assert_eq!(variant, converted_back_variant);
+
+        let variant = Variant::Array(vec![Variant::I4(1), Variant::I4(2)]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant).unwrap();
+
+        assert_eq!(variant, converted_back_variant);
+
+        let variant = Variant::Array(vec![Variant::I8(1), Variant::I8(2)]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant)
+            .unwrap()
+            .convert_into_cim_type(CIM_SINT64)
+            .unwrap();
+
+        assert_eq!(variant, converted_back_variant);
+
+        let variant = Variant::Array(vec![Variant::R8(1.), Variant::R8(2.)]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant).unwrap();
+
+        assert_eq!(variant, converted_back_variant);
+
+        let variant = Variant::Array(vec![Variant::Bool(true), Variant::Bool(false)]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant).unwrap();
+
+        assert_eq!(variant, converted_back_variant);
+
+        let variant = Variant::Array(vec![
+            Variant::String("a".to_string()),
+            Variant::String("b".to_string()),
+        ]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant).unwrap();
+
+        assert_eq!(variant, converted_back_variant);
+
+        // Empty arrays are converted to empty variants.
+        let variant = Variant::Array(vec![]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant).unwrap();
+
+        assert_eq!(converted_back_variant, Variant::Null);
+
+        let variant = Variant::Array(vec![Variant::I1(0), Variant::I1(1)]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant)
+            .unwrap()
+            .convert_into_cim_type(CIM_SINT8)
+            .unwrap();
+        assert_eq!(variant, converted_back_variant);
+
+        let variant = Variant::Array(vec![Variant::R4(0.), Variant::R4(1.)]);
+        let ms_variant = VARIANT::try_from(variant.clone()).unwrap();
+        let converted_back_variant = Variant::from_variant(&ms_variant).unwrap();
+        assert_eq!(variant, converted_back_variant);
+    }
+
+    #[test]
+    fn it_does_not_convert_array_to_unsupported_ms_variant() {
+        let variant = Variant::Array(vec![Variant::String("a".to_string()), Variant::I8(0)]);
+        assert!(
+            VARIANT::try_from(variant.clone()).is_err(),
+            "Mixed arrays are not supported"
+        );
+    }
 }
